
/**
 * MIGRATION GUIDE:
 * ================
 * The IEmailConfig and IMtaConfig interfaces are deprecated.
 * Please use IUnifiedEmailServerOptions from '../mail/routing/classes.unified.email.server.js' instead.
 * 
 * Example migration:
 * 
 * OLD:
 * ```typescript
 * const config: IEmailConfig = {
 *   ports: [25, 587],
 *   hostname: 'mail.example.com',
 *   domainRules: [...],
 *   defaultMode: 'forward'
 * };
 * ```
 * 
 * NEW:
 * ```typescript
 * const config: IUnifiedEmailServerOptions = {
 *   ports: [25, 587],
 *   hostname: 'mail.example.com',
 *   domains: ['example.com'],
 *   domainRules: [...],
 *   defaultMode: 'forward'
 * };
 * ```
 * 
 * The new interface consolidates all email configuration into a single, cleaner structure.
 */

/**
 * Email processing modes
 */
export type EmailProcessingMode = 'forward' | 'mta' | 'process';

/**
 * Domain rule for email routing
 */
export interface IDomainRule {
  /**
   * Pattern to match (e.g., "*@example.com")
   */
  pattern: string;
  
  /**
   * Processing mode
   */
  mode: EmailProcessingMode;
  
  /**
   * Target server for forwarding mode
   */
  target?: {
    /**
     * Target server hostname or IP
     */
    server: string;
    
    /**
     * Target server port
     */
    port?: number;
    
    /**
     * Whether to use TLS for forwarding
     */
    useTls?: boolean;
  };
  
  /**
   * MTA options for mta mode
   */
  mtaOptions?: {
    /**
     * Domain for MTA
     */
    domain?: string;
    
    /**
     * Whether to sign with DKIM
     */
    dkimSign?: boolean;
    
    /**
     * DKIM options
     */
    dkimOptions?: {
      /**
       * Domain name for DKIM
       */
      domainName: string;
      
      /**
       * Key selector
       */
      keySelector: string;
      
      /**
       * Private key
       */
      privateKey?: string;
    };
  };
  
  /**
   * Whether to scan content in process mode
   */
  contentScanning?: boolean;
  
  /**
   * Content scanners to apply
   */
  scanners?: Array<{
    /**
     * Scanner type
     */
    type: 'spam' | 'virus' | 'attachment';
    
    /**
     * Threshold for scanner
     */
    threshold?: number;
    
    /**
     * Action to take
     */
    action: 'tag' | 'reject';
    
    /**
     * Blocked file extensions for attachment scanner
     */
    blockedExtensions?: string[];
  }>;
  
  /**
   * Email transformations to apply
   */
  transformations?: Array<{
    /**
     * Transformation type
     */
    type: 'addHeader';
    
    /**
     * Header name
     */
    header?: string;
    
    /**
     * Header value
     */
    value?: string;
  }>;
}


