/**
 * Email Port Mapping Configuration
 * 
 * Centralizes the logic for mapping external email ports to internal ports
 * when running behind SmartProxy
 */

/**
 * Default email port mapping
 * Maps external ports to internal ports for SmartProxy forwarding
 */
export const DEFAULT_EMAIL_PORT_MAPPING: Record<number, number> = {
  25: 10025,    // SMTP - Standard email delivery port
  587: 10587,   // Submission - Authenticated submission port  
  465: 10465,   // SMTPS - Implicit TLS submission port
  2525: 12525   // Alternative SMTP port (often used for testing)
};

/**
 * Email port configuration
 */
export interface IEmailPortConfig {
  /**
   * External port number
   */
  external: number;
  
  /**
   * Internal port number (where the email server actually listens)
   */
  internal: number;
  
  /**
   * Port description
   */
  description: string;
  
  /**
   * TLS mode for this port
   */
  tlsMode: 'none' | 'starttls' | 'implicit';
  
  /**
   * Whether authentication is required on this port
   */
  requireAuth?: boolean;
}

/**
 * Standard email port configurations
 */
export const EMAIL_PORT_CONFIGS: Record<number, Omit<IEmailPortConfig, 'internal'>> = {
  25: {
    external: 25,
    description: 'SMTP - Standard email delivery',
    tlsMode: 'starttls',
    requireAuth: false
  },
  587: {
    external: 587,
    description: 'Submission - Authenticated email submission',
    tlsMode: 'starttls', 
    requireAuth: true
  },
  465: {
    external: 465,
    description: 'SMTPS - Implicit TLS submission',
    tlsMode: 'implicit',
    requireAuth: true
  },
  2525: {
    external: 2525,
    description: 'Alternative SMTP - Often used for testing',
    tlsMode: 'starttls',
    requireAuth: false
  }
};

/**
 * Map external ports to internal ports
 * @param externalPorts Array of external ports
 * @param customMapping Optional custom port mapping
 * @returns Array of internal ports
 */
export function mapEmailPorts(
  externalPorts: number[], 
  customMapping?: Record<number, number>
): number[] {
  const mapping = { ...DEFAULT_EMAIL_PORT_MAPPING, ...customMapping };
  
  return externalPorts.map(port => {
    // Use custom mapping if available, otherwise add 10000 to the port
    return mapping[port] || port + 10000;
  });
}

/**
 * Get full port configuration including internal port mapping
 * @param externalPort External port number
 * @param customMapping Optional custom port mapping
 * @returns Full port configuration
 */
export function getEmailPortConfig(
  externalPort: number,
  customMapping?: Record<number, number>
): IEmailPortConfig | undefined {
  const config = EMAIL_PORT_CONFIGS[externalPort];
  if (!config) {
    return undefined;
  }
  
  const mapping = { ...DEFAULT_EMAIL_PORT_MAPPING, ...customMapping };
  const internalPort = mapping[externalPort] || externalPort + 10000;
  
  return {
    ...config,
    internal: internalPort
  };
}

/**
 * Generate SmartProxy routes for email ports
 * @param externalPorts Array of external ports to handle
 * @param hostname Hostname for the email server
 * @param customMapping Optional custom port mapping
 * @returns Array of SmartProxy route configurations
 */
export function generateEmailProxyRoutes(
  externalPorts: number[],
  _hostname: string,
  customMapping?: Record<number, number>
): plugins.smartproxy.IRouteConfig[] {
  const routes: plugins.smartproxy.IRouteConfig[] = [];
  
  for (const externalPort of externalPorts) {
    const config = getEmailPortConfig(externalPort, customMapping);
    if (!config) {
      continue;
    }
    
    // Create route for this email port
    const route: plugins.smartproxy.IRouteConfig = {
      name: `email-${config.description.toLowerCase().replace(/\s+/g, '-')}`,
      match: {
        ports: [externalPort]
      },
      action: {
        type: 'forward',
        target: {
          host: 'localhost',
          port: config.internal
        },
        tls: {
          // For SMTP/Submission ports, use passthrough to let email server handle STARTTLS
          // For SMTPS (465), SmartProxy handles TLS termination
          mode: config.tlsMode === 'implicit' ? 'terminate' : 'passthrough',
          certificate: config.tlsMode === 'implicit' ? 'auto' : undefined
        }
      }
    };
    
    routes.push(route);
  }
  
  return routes;
}

/**
 * Validate email port configuration
 * @param ports Array of ports to validate
 * @throws Error if invalid ports are specified
 */
export function validateEmailPorts(ports: number[]): void {
  const invalidPorts = ports.filter(port => {
    // Check if port is a valid number
    if (!Number.isInteger(port) || port < 1 || port > 65535) {
      return true;
    }
    
    // Warn about non-standard ports
    if (!EMAIL_PORT_CONFIGS[port]) {
      console.warn(`Port ${port} is not a standard email port. Supported standard ports are: ${Object.keys(EMAIL_PORT_CONFIGS).join(', ')}`);
    }
    
    return false;
  });
  
  if (invalidPorts.length > 0) {
    throw new Error(`Invalid email ports specified: ${invalidPorts.join(', ')}`);
  }
}

// Type imports to avoid circular dependencies
import type * as plugins from '../plugins.js';